DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

ALLOWED_HOSTS = [
    "example.com",
]

INSTALLED_APPS = [
    "django.contrib.sites",
    "cjk404",
]

SECRET_KEY = "notimportant"

APPEND_SLASH = False

MIDDLEWARE = ["cjk404.middleware.PageNotFoundRedirectMiddleware"]

SITE_ID = 1

# Django seems to require a ROOT_URLCONF.
ROOT_URLCONF = __name__
urlpatterns = []



# Generated by Django 3.2.3 on 2021-08-30 10:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
        ('cjk404', '0003_pagenotfoundentry_regular_expression'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='hits',
            field=models.PositiveIntegerField(default=1, verbose_name='Number of Hits (Page Views)'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='last_hit',
            field=models.DateTimeField(verbose_name='Last Hit (Last Page View)'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='redirect_to_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.page', verbose_name='Redirect To (Wagtail Page)'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='redirect_to_url',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='New Path / Redirect To (URL)'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='regular_expression',
            field=models.BooleanField(default=False, verbose_name='Regular Expression'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pagenotfound_entries', to='wagtailcore.site', verbose_name='Website / Domain'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='url',
            field=models.CharField(max_length=200, verbose_name='Old Path / Redirect From (URL)'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-11 20:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjk404', '0005_auto_20211011_2131'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='hits',
            field=models.PositiveIntegerField(default=0, verbose_name='# Hits'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='last_hit',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Last Hit'),
        ),
    ]



from django.db import models
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel, PageChooserPanel
from wagtail.core.models import Page, Site


class PageNotFoundEntry(models.Model):
    site = models.ForeignKey(
        Site,
        related_name="pagenotfound_entries",
        on_delete=models.CASCADE,
        verbose_name="Site",
    )

    url = models.CharField(max_length=200, verbose_name="Redirect from URL")
    redirect_to_url = models.CharField(
        max_length=200,
        null=True,
        blank=True,
        verbose_name="Redirect to URL",
    )
    redirect_to_page = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        verbose_name="Redirect to Page",
    )

    created = models.DateTimeField(
        auto_now_add=True, blank=True, verbose_name="Created"
    )
    last_hit = models.DateTimeField(
        auto_now_add=True, blank=True, verbose_name="Last Hit"
    )
    hits = models.PositiveIntegerField(default=0, verbose_name="# Hits")
    permanent = models.BooleanField(default=False)

    regular_expression = models.BooleanField(default=False, verbose_name="RegExp")

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("site"),
                FieldPanel("url"),
                FieldPanel("regular_expression"),
            ],
            heading="Old Path / Redirect From",
        ),
        MultiFieldPanel(
            [
                FieldPanel("hits"),
            ],
            heading="Hit stats",
            classname="collapsible",
        ),
        MultiFieldPanel(
            [
                PageChooserPanel("redirect_to_page"),
                FieldPanel("redirect_to_url"),
                FieldPanel("permanent"),
            ],
            heading="New Path / Redirect To",
            classname="collapsible",
        ),
    ]

    @property
    def redirect_to(self):
        if self.redirect_to_page:
            return self.redirect_to_page.url
        return self.redirect_to_url

    def __str__(self):
        return f"{self.url} ---> {self.redirect_to}"

    class Meta:
        verbose_name_plural = "page not found redirects"
        ordering = ("-hits",)



# Generated by Django 3.2.7 on 2021-10-11 19:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
        ('cjk404', '0004_auto_20210830_1007'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='hits',
            field=models.PositiveIntegerField(default=1, verbose_name='# Hits'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='last_hit',
            field=models.DateTimeField(verbose_name='Last Hit'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='redirect_to_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.page', verbose_name='Redirect to Page'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='redirect_to_url',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Redirect to URL'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='regular_expression',
            field=models.BooleanField(default=False, verbose_name='RegExp'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pagenotfound_entries', to='wagtailcore.site', verbose_name='Site'),
        ),
        migrations.AlterField(
            model_name='pagenotfoundentry',
            name='url',
            field=models.CharField(max_length=200, verbose_name='Redirect from URL'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-10 14:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
    ]

    operations = [
        migrations.CreateModel(
            name='PageNotFoundEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=200)),
                ('redirect_to_url', models.CharField(blank=True, max_length=200, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_hit', models.DateTimeField()),
                ('hits', models.PositiveIntegerField(default=1)),
                ('permanent', models.BooleanField(default=False)),
                ('redirect_to_page', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.page')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pagenotfound_entries', to='wagtailcore.site')),
            ],
            options={
                'verbose_name_plural': 'page not found redirects',
                'ordering': ('-hits',),
            },
        ),
    ]



# Generated by Django 4.0.7 on 2022-08-10 20:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('cjk404', '0001_initial'), ('cjk404', '0002_alter_pagenotfoundentry_id'), ('cjk404', '0003_pagenotfoundentry_regular_expression'), ('cjk404', '0004_auto_20210830_1007'), ('cjk404', '0005_auto_20211011_2131'), ('cjk404', '0006_auto_20211011_2219')]

    initial = True

    dependencies = [
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
    ]

    operations = [
        migrations.CreateModel(
            name='PageNotFoundEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=200, verbose_name='Redirect from URL')),
                ('redirect_to_url', models.CharField(blank=True, max_length=200, null=True, verbose_name='Redirect to URL')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('last_hit', models.DateTimeField(auto_now_add=True, verbose_name='Last Hit')),
                ('hits', models.PositiveIntegerField(default=0, verbose_name='# Hits')),
                ('permanent', models.BooleanField(default=False)),
                ('redirect_to_page', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.page', verbose_name='Redirect to Page')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pagenotfound_entries', to='wagtailcore.site', verbose_name='Site')),
                ('regular_expression', models.BooleanField(default=False, verbose_name='RegExp')),
            ],
            options={
                'verbose_name_plural': 'page not found redirects',
                'ordering': ('-hits',),
            },
        ),
    ]



from django.apps import AppConfig


class Managed404Config(AppConfig):
    """Forked from wagtail_managed404, which was abandoned in 2018"""

    default_auto_field = "django.db.models.BigAutoField"
    name = "cjk404"



# Generated by Django 3.2.3 on 2021-08-30 09:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjk404', '0002_alter_pagenotfoundentry_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='pagenotfoundentry',
            name='regular_expression',
            field=models.BooleanField(default=False),
        ),
    ]



import re
from sys import stdout

from django import http
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect
from django.utils.timezone import now

from .models import PageNotFoundEntry
from wagtail.core.models import Site

IGNORED_404S = getattr(settings, "IGNORED_404S", [r"^/static/", r"^/favicon.ico"])

DJANGO_REGEX_REDIRECTS_CACHE_KEY = "django-regex-redirects-regular"
DJANGO_REGEX_REDIRECTS_CACHE_REGEX_KEY = "django-regex-redirects-regex"
DJANGO_REGEX_REDIRECTS_CACHE_TIMEOUT = 60


class PageNotFoundRedirectMiddleware:
    def __init__(self, response):
        self.response = response
        self.blacklist_url_patterns = [re.compile(string) for string in IGNORED_404S]

    def __call__(self, request):
        url = request.path
        if self._check_url_in_blacklist(url):
            return self.response(request)
        else:
            return self.handle_request(request)

    def _check_url_in_blacklist(self, url):
        return any(pattern.match(url) for pattern in self.blacklist_url_patterns)

    def updateHitCount(self, entry_id: int):
        entry = PageNotFoundEntry.objects.get(id=entry_id)
        entry.hits += 1
        entry.last_hit = now()
        entry.save()

    def host_with_protocol(self, request):
        http_host = request.META.get("HTTP_HOST", "")
        if http_host:
            if request.is_secure():
                http_host = f"https://{http_host}"
            else:
                http_host = f"http://{http_host}"
        return http_host

    def HttpRedirect301302(self, request, location, is_permanent=False):
        if not location:
            return self.response(request)

        http_host = self.host_with_protocol(request)

        if not (location.startswith("http") or location.startswith("https")):
            location = http_host + location
        if is_permanent:
            return HttpResponsePermanentRedirect(location)
        else:
            return HttpResponseRedirect(location)

    def handle_request(self, request):
        response = self.response(request)
        if response.status_code != 404:
            return response

        url = request.path
        site = Site.find_for_request(request)

        # find matching url in PageNotFoundEntry, and increase hit count

        full_path = request.get_full_path()

        redirects = cache.get(DJANGO_REGEX_REDIRECTS_CACHE_KEY)
        if redirects is None:
            redirects = list(PageNotFoundEntry.objects.all().values())
            cache.set(
                DJANGO_REGEX_REDIRECTS_CACHE_KEY,
                redirects,
                DJANGO_REGEX_REDIRECTS_CACHE_TIMEOUT,
            )

        # non-regexp to be attempted first (faster)
        for redirect in redirects:

            if redirect["url"] == full_path:
                self.updateHitCount(redirect["id"])

                return self.HttpRedirect301302(
                    request, redirect["redirect_to_url"], redirect["permanent"]
                )

            if settings.APPEND_SLASH and not request.path.endswith("/"):
                path_len = len(request.path)
                slashed_full_path = f"{full_path[:path_len]}/{full_path[path_len:]}"
                stdout.write(f"SFP: {slashed_full_path}")

                if redirect["url"] == slashed_full_path:
                    self.updateHitCount(redirect["id"])
                    return self.HttpRedirect301302(
                        request, redirect["redirect_to_url"], redirect["permanent"]
                    )

        # no match found, try regexp
        regular_expressions_redirects = cache.get(
            DJANGO_REGEX_REDIRECTS_CACHE_REGEX_KEY
        )
        if regular_expressions_redirects is None:
            regular_expressions_redirects = list(
                PageNotFoundEntry.objects.filter(regular_expression=True).values()
            )
            cache.set(
                DJANGO_REGEX_REDIRECTS_CACHE_REGEX_KEY,
                regular_expressions_redirects,
                DJANGO_REGEX_REDIRECTS_CACHE_TIMEOUT,
            )

        for redirect in regular_expressions_redirects:

            try:
                old_path = re.compile(redirect["url"], re.IGNORECASE)
            except re.error:
                continue

            if old_path.match(full_path):

                self.updateHitCount(redirect["id"])

                new_path = redirect["redirect_to_url"].replace("$", "\\")
                replaced_path = re.sub(old_path, new_path, full_path)
                return self.HttpRedirect301302(
                    request, replaced_path, redirect["permanent"]
                )

        if response.status_code == 404:
            if not PageNotFoundEntry.objects.filter(url=url, site=site).exists():
                PageNotFoundEntry.objects.create(site=site, url=url, hits=1)
        return response



from __future__ import unicode_literals

from unittest.case import skipUnless

from django.conf import settings
from django.test import TestCase
from django.test.utils import override_settings
from django.core.cache import cache
from wagtail.core.models import Site


from .middleware import (
    DJANGO_REGEX_REDIRECTS_CACHE_KEY,
    DJANGO_REGEX_REDIRECTS_CACHE_REGEX_KEY,
)
from .models import PageNotFoundEntry


class Cjk404RedirectTests(TestCase):
    def setUp(self):
        cache.delete(DJANGO_REGEX_REDIRECTS_CACHE_KEY)
        cache.delete(DJANGO_REGEX_REDIRECTS_CACHE_REGEX_KEY)

    def test_model(self):
        site = Site.objects.filter(is_default_site=True)[0]
        r1 = PageNotFoundEntry.objects.create(
            url="/initial", redirect_to_url="/new_target", site=site
        )
        self.assertEqual(r1.__str__(), "/initial ---> /new_target")

    def redirect_url(
        self,
        permanent,
        old_url,
        redirect_url,
        requested_url,
        status_code=None,
        regexp=False,
    ):
        site = Site.objects.filter(is_default_site=True)[0]
        pnfe = PageNotFoundEntry.objects.create(
            permanent=permanent,
            url=old_url,
            redirect_to_url=redirect_url,
            site=site,
        )
        self.assertEqual(pnfe.hits, 0)
        response = self.client.get(requested_url)
        if status_code:
            self.assertRedirects(
                response, redirect_url, status_code=status_code, target_status_code=404
            )
        else:
            self.assertRedirects(response, redirect_url)
        pnfe.refresh_from_db()
        self.assertEqual(pnfe.hits, 1)

    def test_redirect(self):
        self.redirect_url(False, "/initial1/", "/new_target/", "/initial1/", 302)

    def test_redirect_premanent(self):
        self.redirect_url(True, "/initial2/", "/new_target/", "/initial2/", 301)

    # def test_regular_expression(self):
    #     self.redirect_url(
    #         False,
    #         "/news/index/[a-z]/",
    #         "/my/news/$1/",
    #         "/news/index/b/",
    #         302,
    #         True,
    #     )

    #
    # def test_fallback_redirects(self):
    #     """
    #     Ensure redirects with fallback_redirect set are the last evaluated
    #     """
    #     PageNotFoundEntry.objects.create(
    #         old_path='/project/foo',
    #         new_path='/my/project/foo')
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/project/foo/(.*)',
    #         new_path='/my/project/foo/$1',
    #         regular_expression=True)
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/project/(.*)',
    #         new_path='/projects',
    #         regular_expression=True,
    #         fallback_redirect=True)
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/project/bar/(.*)',
    #         new_path='/my/project/bar/$1',
    #         regular_expression=True)
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/project/bar',
    #         new_path='/my/project/bar')
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/second_project/.*',
    #         new_path='http://example.com/my/second_project/bar/',
    #         regular_expression=True)
    #
    #     PageNotFoundEntry.objects.create(
    #         old_path='/third_project/(.*)',
    #         new_path='http://example.com/my/third_project/bar/$1',
    #         regular_expression=True)
    #
    #     response = self.client.get('/project/foo')
    #     self.assertRedirects(response,
    #                          '/my/project/foo',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/project/bar')
    #     self.assertRedirects(response,
    #                          '/my/project/bar',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/project/bar/details')
    #     self.assertRedirects(response,
    #                          '/my/project/bar/details',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/project/foobar')
    #     self.assertRedirects(response,
    #                          '/projects',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/project/foo/details')
    #     self.assertRedirects(response,
    #                          '/my/project/foo/details',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/second_project/details')
    #     self.assertRedirects(response,
    #                          'http://example.com/my/second_project/bar/',
    #                          status_code=301, target_status_code=404)
    #
    #     response = self.client.get('/third_project/details')
    #     self.assertRedirects(response,
    #                          'http://example.com/my/third_project/bar/details',
    #                          status_code=301, target_status_code=404)



from wagtail.contrib.modeladmin.helpers import PermissionHelper
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register

from .models import PageNotFoundEntry


class PageNotFoundPermissionHelper(PermissionHelper):
    def user_can_create(self, user):
        return True


@modeladmin_register
class PageNotFoundEntryAdmin(ModelAdmin):
    permission_helper_class = PageNotFoundPermissionHelper
    model = PageNotFoundEntry
    menu_label = "404 Redirects"
    menu_icon = "redirect"
    list_display = (
        "url",
        "redirect_to_url",
        "redirect_to_page",
        "hits",
        "permanent",
        "last_hit",
        "regular_expression",
    )
    list_filter = ("permanent", "regular_expression", "site")
    search_fields = ("url", "redirect_to_url")
